package nervoussystem.nurbs;

import processing.core.*;

public class Vector4D {
	  public float x,y,z,w;
	  //float t;
	  public Vector4D() {
	    x=0;
	    y=0;
	    z=0;
	    w=0;
	  }
	  public Vector4D(float _x,float _y,float _z) {
	    x = _x;
	    y = _y;
	    z = _z;
	    w=1;
	  }
	  
	  public Vector4D(float _x,float _y,float _z, float _w) {
	    x = _x;
	    y = _y;
	    z = _z;
	    w = _w;    
	  }
	  
	  public Vector4D(PVector pt, float _w) {
		  w = _w;
		  x = pt.x*w;
		  y = pt.y*w;
		  z = pt.z*w;
	  }
	  
	  public Vector4D(Vector4D v) {
	    x = v.x;
	    y = v.y;
	    z = v.z;
	    w = v.w;
	  }
	  
	  public Vector4D(PVector v) {
	    x = v.x;
	    y = v.y;
	    z = v.z;
	    w = 1;
	  }
	  
	  public PVector projectDown() {
		  if(w == 0) return new PVector();
		  return new PVector(x/w,y/w,z/w);
	  }
	  
	  public float length() {
	    return (float) Math.sqrt(x*x+y*y+z*z+w*w);
	  }
	  
	  public Vector4D setLength(float newL) {
	    float l = this.length();
	    if (l==0) return this;
	    x = x*newL/l;
	    y = y*newL/l;
	    z = z*newL/l;
	    w = w*newL/l;
	    return this;
	  }
	  
	  public String toString() {
	    return x + " " + y + " " + z + " " + w;
	  }


	public static Vector4D add(Vector4D v1, Vector4D v2) {
	  return new Vector4D(v1.x+v2.x, v1.y+v2.y,v1.z+v2.z,v1.w+v2.w);
	}

	public static Vector4D add(Vector4D v1, float val) {
	  return new Vector4D(v1.x+val, v1.y+val, v1.z+val,v1.w+val);
	}

	public static Vector4D sub(Vector4D v1, Vector4D v2) {
	  return new Vector4D(v1.x-v2.x,v1.y-v2.y,v1.z-v2.z,v1.w-v2.w);
	}

	public static Vector4D multiply(Vector4D v1, float val) {
	  return new Vector4D(v1.x*val, v1.y*val, v1.z*val,v1.w*val);
	}

	
	public static float dot(Vector4D v1, Vector4D v2) {
	  return v1.x*v2.x+v1.y*v2.y+v1.z*v2.z+v1.w*v2.w;
	}
}